import java.io.*;
import java.awt.*;
import java.awt.event.*;


public class Graph2d extends Frame implements ActionListener
{
	
	TwoDMural drawArea;

	final int width=800,height=550;
	//final int shapeCount=3;
	//final int transCount=2;
	
	MenuBar mb=new MenuBar();
	Menu menus[]=new Menu[2];
	
	int curShape;
	int curIndex;
	
	String shapePrompt[]={"Line","Circle","Polygon","Clip"};
	String transPrompt[]={"Rotate","Translate","MirrorX","MirrorY"};
	int shapeValue[]={Shape2d.LINE,Shape2d.CIRCLE,Shape2d.POLYGON,Shape2d.CLIP};
	int transShortcut[]={KeyEvent.VK_R,KeyEvent.VK_T,KeyEvent.VK_X,KeyEvent.VK_Y};
	int shapeShortcut[]={KeyEvent.VK_L,KeyEvent.VK_C,KeyEvent.VK_P,KeyEvent.VK_S};
	
	Label statusLabel=new Label();	
	
	List shapeList=new List();
	
	TextField transXText,transYText;
	TextField rotAngText;
	Label transXLabel=new Label("X");
	Label transYLabel=new Label("Y");
	Label rotAngLabel=new Label("Angle");

	
	Graph2d(Frame w)
	{
		//super(w,"TwoD Graphics",true);
		super("TwoD Graphics");
		addWindowListener(new WinAdap());

		setMenuBar(mb);
		setSize(width,height);
		setResizable(false);

		initCockpit(); //Menus
		
		shapeList.addActionListener(this);
		
		statusLabel.setText("Shape :"+shapePrompt[curShape]);
		

		//This is the drawing surface
		drawArea=new TwoDMural(shapeList);
		drawArea.setBackground(Color.black);
		drawArea.orgX=width/2-50;
		drawArea.orgY=height/2;
	
		transXText=new TextField("0");
		transYText=new TextField("0");
		rotAngText=new TextField("0");
		
		//Adding the texts for x,y and angle
		Panel transTexts=new Panel();
		transTexts.setLayout(new GridLayout(3,2));
		transTexts.add(transXLabel);
		transTexts.add(transYLabel);
		transTexts.add(transXText);
		transTexts.add(transYText);
		transTexts.add(rotAngLabel);
		transTexts.add(rotAngText);
		
		Panel leftPanel=new Panel();
		leftPanel.setLayout(new BorderLayout());
		leftPanel.add("Center",shapeList);
		leftPanel.add("South",transTexts);

		add("North",statusLabel);
		add("Center",drawArea);
		add("West",leftPanel);
		
				
		curIndex=-1;
		
		setVisible(true);
	}
	
	public	void initCockpit()
	{
		int i;
		menus[0]=new Menu("Shapes");
		for(i=0;i<shapePrompt.length;++i)
			setMenuItem(menus[0],shapePrompt[i],shapeShortcut[i]);
		
		curShape=0;
	
		menus[1]=new Menu("Transforms");
		for(i=0;i<transPrompt.length;++i)
			setMenuItem(menus[1],transPrompt[i],transShortcut[i]);
		for(i=0;i<menus.length;++i)
			mb.add(menus[i]);
	}
	
/*
	public void setCheckMenuItem(Menu m,String s,boolean f)
	{
		CheckboxMenuItem mi=new CheckboxMenuItem(s,f);
		m.add(mi);
		mi.addActionListener(this);
		mi.setState(f);
	}
*/
	public void setMenuItem(Menu m,String s,int key)
	{
		MenuItem mi=new MenuItem(s,new MenuShortcut(key));
		m.add(mi);
		mi.addActionListener(this);
	}

	public void actionPerformed(ActionEvent ae)
	{
		int i;
		
		//Current shape changed
		for(i=0;i<shapePrompt.length;++i)
		{
			if(ae.getActionCommand().equals(shapePrompt[i]))
			{
				drawArea.curShape=shapeValue[i];
				statusLabel.setText("Current Shape :"+shapePrompt[i]);
				return;
			}
			
		}
		
		
		//Shapes list doubleclicked
		//so change selected line
		if(ae.getSource()==shapeList)
		{
			Graphics g=drawArea.getGraphics();
			if(curIndex!=-1)
			{
				drawArea.selUnselShape(curIndex); // Selshape erases or draws (XOR mode)
				drawArea.draw(curIndex);
			}
			curIndex=shapeList.getSelectedIndex();
			drawArea.curSel=curIndex;				
			if(curIndex!=-1)
			{
				drawArea.selUnselShape(curIndex);
				drawArea.curSel=curIndex;				
			}
			return ;
		}
		
		//Rotate
		if(ae.getActionCommand().equals("Rotate"))
		{
			try
			{
				int rotang=Integer.parseInt(rotAngText.getText());
				int tx=Integer.parseInt(transXText.getText());
				int ty=Integer.parseInt(transYText.getText());
				if(curIndex!=-1)
				{
					//drawArea.erase(curIndex);
					drawArea.shapes[curIndex].rotate(rotang,tx,ty);
					//drawArea.draw(curIndex);
					//drawArea.selUnselShape(curIndex);
					drawArea.repaint();
				}
			}
			catch(Exception e)
			{
				return ;
			}
			
			return;
		}
		//Translate
		if(ae.getActionCommand().equals("Translate"))
		{
			try
			{
				int tx=Integer.parseInt(transXText.getText());
				int ty=Integer.parseInt(transYText.getText());
				if(curIndex!=-1)
				{
					//drawArea.erase(curIndex);
					drawArea.shapes[curIndex].translate(tx,ty);
					//drawArea.draw(curIndex);
					//drawArea.selUnselShape(curIndex);
					drawArea.repaint();
				}
			}
			catch(Exception e)
			{
				return ;
			}
			
			return;
		}
		//Mirrorx
		if(ae.getActionCommand().equals("MirrorX"))
		{
			if(curIndex!=-1)
			{
				//drawArea.erase(curIndex);
				drawArea.shapes[curIndex].mirrorX();
				//drawArea.draw(curIndex);
				//drawArea.selUnselShape(curIndex);
				drawArea.repaint();
			}
			return;
		}
		//MirrorY
		if(ae.getActionCommand().equals("MirrorY"))
		{
			if(curIndex!=-1)
			{
				//drawArea.erase(curIndex);
				drawArea.shapes[curIndex].mirrorY();
				//drawArea.draw(curIndex);
				//drawArea.selUnselShape(curIndex);
				drawArea.repaint();
			}
			return;
		}
	}

	public class WinAdap extends WindowAdapter
	{	
		public void windowClosing(WindowEvent e)
		{
			hide();
		}
	}
}